-- "Sunrise" -- a simplified version of wigglers

module Sunrise where

import Monad
import Pan
import Simple (swirlUI)

sunshine :: BoolE -> FractionE -> Region -> FractionE -> ImageC
sunshine outerCropped ir reg darkness =
  crop cropReg
       ( (\ z -> lerpC z yellow (lerpC darkness blue black))
       . (\ r -> (r-ir)/(1-ir))
       . distO
       )
 where
   cropReg = ifE outerCropped udisk universeR
             `intersectR` complementR (disk ir)
             `intersectR` reg
                    

sunshineUI :: UI ImageC
sunshineUI = liftM4 sunshine
               (tick "outer cropped" True)
               (slider "inner radius" (0,1) 0.2)
               wedgesUI
               (slider "darkness" (0,1) 0.2)

wedgesUI :: UI Region
wedgesUI = liftM2 wedges
             (islider "arms"     (0,60) 40)
             (slider  "coverage" (0,1)  0.9)

-- To do: factor common pattern out of wedges, whirl.

sunriseUI :: UI ImageC
sunriseUI = do vshift <- slider "vertical shift" (-100,50) (-50)
               depth  <- slider "depth" (0,1.5) 0.5
               sun    <- sunshineUI
               return $
                 translate (0,vshift) $
                 rippleRadius 1 depth $
                 uscale 50 $
                 sun


swirlWedgesUI :: UI ImageC
swirlWedgesUI = swirlUI `ap`
                liftM2 sun wedgesUI (slider "inner radius" (0,1) 0.2)
 where
   sun reg ir = uscale 50 $ sunshine true ir reg 0


----------------------------------------

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileImageFor bends "sunrise"      sunriseUI
  compileImageFor bends "swirl wedges" swirlWedgesUI
