// PanISlider.cpp: implementation of the CPanISlider class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PanISlider.h"
#include <windowsx.h>
#include <string.h>
#include <tchar.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPanISlider::CPanISlider ( HWND            hWndParent
                         , LPCTSTR         label
                         , int             min
                         , int             max
                         , IntUpdater*     pUpd
                         , void*           pParam
                         , CInvalidator*   pInvalid
    )
{
    TCHAR valStr[255];

    m_min = min;
    m_max = max;
    m_updateValue = pUpd;
    m_pParam = pParam;
    m_iProc = pInvalid;
    m_inFlux = false;

    // Notice that we can't use the default value, initVal, 
    // here for m_value here, since the component may have
    // been instantiated from a settings (.pan) file,
    // in which case the current value is likely to
    // differ from the default. So, we instead use the
    // GetInt() method on the component to get the value
    // to display in the UI.
    //
    m_value = m_updateValue->GetInt(m_pParam);
    _stprintf(valStr, TEXT("%d"), (int)m_value);
 
    Create(hWndParent, label, valStr, 100,20);
 
    SendMessage(GetDlgItem(ID_TRACKBAR), TBM_SETRANGE, 0, MAKELONG(min,max));
    ::Edit_SetText(GetDlgItem(ID_EDIT_FIELD), valStr); 

}

CPanISlider::~CPanISlider()
{

}


//////////////////////////////////////////////////////////////////////
// Event handlers
//////////////////////////////////////////////////////////////////////
LRESULT
CPanISlider::OnHScroll(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	DWORD ev = LOWORD(wParam);
      
	if ( ev == TB_ENDTRACK || ev == TB_THUMBTRACK || ev == TB_THUMBPOSITION) {
	  TCHAR msg[80];
	  int val;
	
	  val = SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
	  _stprintf(msg, TEXT("%d"), val);

      m_value = val;

	  // update the edit field with new value.
	  m_inFlux = TRUE;
      ::Edit_SetText(GetDlgItem(ID_EDIT_FIELD), msg); 
	  m_inFlux = FALSE;

	  // .. and the view window
      m_updateValue->UpdateInt(m_pParam, val);
	  m_iProc->Invalidate();
    }

    return 0;
}


LRESULT
CPanISlider::OnChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    TCHAR valStr[255];
    int val, pos;
      
    if ( m_inFlux != TRUE ) {
        // the trackbar is not being dragged, so this event was caused
	    // by the user modifying the edit box value directly.
        ::Edit_GetText(hWndCtl, valStr, 255);
#ifdef _UNICODE
        val = (int)_wtoi(valStr);
#else
        val = (int)atoi(valStr);
#endif
        m_value = val;
	
        m_updateValue->UpdateInt(m_pParam,
                                 val // min (m_max, max(m_min, val))
            );
	    m_iProc->Invalidate();
    }

    pos = SendMessage(hWndCtl, TBM_GETPOS, 0, 0);

    if ( m_inFlux != true && val != pos ) {
        // If new value was typed in & is different to current,
        // then we'll need to update the trackbar thumb.
        // Don't force values into range.
        // val = (val > m_max ? m_max : val);
        // val = (val < m_min ? m_min : val);
        SendMessage(GetDlgItem(ID_TRACKBAR), TBM_SETPOS, TRUE, val);
    }

    return 0;
}

