// PanTextBox.cpp: implementation of the CPanTextBox class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PanTextBox.h"
#include <windowsx.h>

#define ID_TEXTBOX_EDIT 12

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define MAX_TEXTBOX_LENGTH  512

CPanTextBox::CPanTextBox
	    ( HWND              hWndParent
	    , LPCTSTR           label
	    , LPCTSTR           defValue
	    , int               nWidth
	    , int               nHeight
        , IntUpdater*       pUpd
        , void*             pParamBits
        , void*             pParamWidth
        , void*             pParamHeight
        , CInvalidator* iProc
	    )
 //       :CBox(false)
{

    m_updateValue   = pUpd;
    m_pParamBits    = pParamBits;
    m_pParamWidth   = pParamWidth;
    m_pParamHeight  = pParamHeight;
    m_iProc         = iProc;
    m_pData         = NULL;

    m_isVertical = false;

//    m_nWidth  = 0;
//    m_nHeight = 0;

    m_text = new TCHAR[MAX_TEXTBOX_LENGTH];

    // the initial font to use is the system font;
    // a settings file may override that.
    //
    // Just using the system font as the default is a little
    // bit pessimistic, as we can probably assume that 
    // boxes will now have a basic set of TT fonts installed.
    HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);

    GetObject(hFont,sizeof(LOGFONT),&m_lf); // no need to delete stock object

    _tcscpy(m_text,defValue);

    Create(hWndParent, label, nWidth, nHeight, -1);
}

CPanTextBox::~CPanTextBox()
{
    delete m_text;
}

HWND
CPanTextBox::Create
		( HWND     hWndParent
		, LPCTSTR  label
		, int nWidth
        , int nHeight
		, int selectorID
		)
{
    HWND  hWndButton, hWndEdit;
    RECT r;
    SIZE* pSize;

    r.top = r.left = 0;
    r.bottom = nHeight;
    r.right = nWidth - 6;

    CBox::Create(label, 150,50, hWndParent);

    m_edit = new CEditBox();
    hWndEdit = m_edit->Create(*this, r, m_text,0,0,ID_TEXTBOX_EDIT);

    r.top = r.left = 0;
    r.bottom = 25;
    r.right = 50;

    m_btn = new CBtn();
    hWndButton = m_btn->Create(*this, r, _T("Font.."), 0,0, -1);

    // indicate that neither the edit control nor the button should
    // be resized vertically by the box.
    SetProp(hWndEdit,   RESIZE_PROPERTY_VERT,  (HANDLE)RESIZE_NONE);
    SetProp(hWndButton, RESIZE_PROPERTY_VERT,  (HANDLE)RESIZE_NONE);
    SetProp(hWndEdit,   RESIZE_PROPERTY_HORIZ, (HANDLE)RESIZE_NORMAL);
  
    //
    // Store original sizes so that we can resize correctly later on.
    //

    pSize = (SIZE*)malloc(sizeof(SIZE));
    if (pSize) {
        RECT r;

        ::GetWindowRect(hWndEdit, &r);

        pSize->cx = r.right - r.left;
        pSize->cy = r.bottom - r.top;
        SetProp(hWndEdit, RESIZE_PROPERTY_ORIGSIZE, (HANDLE)pSize);
    }
    
  
    // Finished adding components, lay them out.
    CalcLayout();

    RenderText(m_text, NULL);

    return m_hWnd;

}

//////////////////////////////////////////////////////////////////////
// Event handlers
//////////////////////////////////////////////////////////////////////

LRESULT CPanTextBox::OnClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
     // Time to pop up the FileOpen dialog.
    CHOOSEFONT  cf;
    HFONT       hFont;
    TCHAR       textStr[256];
    int         i = 0;

    ZeroMemory(&cf,sizeof(cf));
    cf.lStructSize = sizeof (CHOOSEFONT);
    cf.hwndOwner = GetDesktopWindow();
    cf.lpLogFont = &m_lf;
    cf.rgbColors = m_cRef;
    cf.Flags = CF_SCREENFONTS | CF_EFFECTS | CF_INITTOLOGFONTSTRUCT;

    // We're now ready to ask for the font:
    if ( ChooseFont(&cf) == TRUE ) {
        m_cRef = cf.rgbColors;
        hFont = CreateFontIndirect(cf.lpLogFont);
        ::Edit_GetText(GetDlgItem(ID_TEXTBOX_EDIT), textStr, 255);
        RenderText(textStr, hFont);
    }
    bHandled = false;
    return 0;    
}


//
// FUNCTION: RenderText()
//
// PURPOSE:  Render text into a bitmap and copy the
//           bits out into sep memory, treating the
//           rendered text just like we treat a bitmap.
//
void
CPanTextBox::RenderText ( LPCTSTR text , HFONT hFont)
{
  SIZE sz;
  BOOL b;
  HDC  hDC;
  HFONT hFontOld;
  int  sLen = _tcslen(text);
 
  // Update the HFONT to mirror currently selected font.
  if (hFont != NULL) {
    DeleteObject(m_hFont);
    m_hFont = hFont;
  }

  // Want to render the text into a bitmap, and pass the
  // resulting bits to the underlying app.
  //
  // Start this off, by creating a DC and selecting the
  // current font into it;
  hDC = CreateCompatibleDC(GetDC());
  hFontOld = (HFONT)SelectObject(hDC, m_hFont);
//  SetBkMode(hDC, OPAQUE);
  

  b = GetTextExtentPoint32(hDC,text, sLen, &sz);
  
  if (b) {
  
    if ( sz.cx != m_nWidth || sz.cy != m_nHeight ) {
      // The size of the rendered text is different from the
      // one previously rendered, so de-allocate the cached HBITMAP
      // and allocate new one (ditto for memory holding the 
      // RGBQUADs.)
      m_nWidth  = sz.cx;
      m_nHeight = sz.cy;
      if (m_hBitmap != NULL) {
	    DeleteObject(m_hBitmap);
      }
	  m_hBitmap = CreateCompatibleBitmap(GetDC(), m_nWidth, m_nHeight);

	  if (m_pData) {
	    free(m_pData);
	  }
	  m_pData = (RGBQUAD*)malloc(sizeof(RGBQUAD)*m_nWidth*m_nHeight);
    }
	
 	HBITMAP hOldBmp = (HBITMAP)SelectObject (hDC, m_hBitmap);

    SetTextColor(hDC, m_cRef);
	TextOut(hDC,0,0,text, sLen);

    m_updateValue->UpdateInt(m_pParamWidth,  m_nWidth);
    m_updateValue->UpdateInt(m_pParamHeight, m_nHeight);
	
    // Copy the rendered text from the HBITMAP
    // to a chunk of RGBQUADs (allocated and maintained
    // by this control.)
	BITMAPINFOHEADER bmih;
    BITMAPINFO bmi;
	
	bmih.biSize     = sizeof(BITMAPINFOHEADER);
    bmih.biWidth    = m_nWidth;
    bmih.biHeight   = m_nHeight;
    bmih.biPlanes   = 1;
    bmih.biBitCount = 32;
    bmih.biCompression = BI_RGB;
    bmih.biSizeImage = 0;
    bmih.biXPelsPerMeter = 0;
    bmih.biYPelsPerMeter = 0;
    bmih.biClrUsed = 0;
    bmih.biClrImportant = 0;

    ZeroMemory(&bmi,sizeof(BITMAPINFO));
    bmi.bmiHeader = bmih;

    SelectObject(hDC, hOldBmp);
    GetDIBits(GetDC(), m_hBitmap, 0, m_nHeight, m_pData, &bmi, DIB_RGB_COLORS);
    m_updateValue->UpdateInt(m_pParamBits, (int)m_pData); 
  }
  SelectObject(hDC, hFontOld);
  DeleteObject(hDC);
  m_iProc->Invalidate();
}

LRESULT CPanTextBox::OnChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    ::Edit_GetText(GetDlgItem(ID_TEXTBOX_EDIT), m_text, MAX_TEXTBOX_LENGTH-1);
	RenderText(m_text, NULL);
	return 0;
}

LPCTSTR CPanTextBox::GetString()
{
    return m_text;
}

void CPanTextBox::SetString(LPCTSTR str)
{
    _tcsncpy(m_text,str,MAX_TEXTBOX_LENGTH-1);
    ::Edit_SetText(GetDlgItem(ID_TEXTBOX_EDIT), m_text);
}

void CPanTextBox::SetFontInfo(FontInfo fI)
{
    memcpy(&m_lf,&fI.lf,sizeof(LOGFONT));
    m_cRef = fI.col;
    RenderText(m_text,CreateFontIndirect(&m_lf));
}

void CPanTextBox::GetFontInfo(FontInfo* pFontInfo)
{
    memcpy(&pFontInfo->lf,&m_lf,sizeof(LOGFONT));
    pFontInfo->col = m_cRef;
}
